package dev.fitko.fitconnect.api.domain.sender;

import dev.fitko.fitconnect.api.FitConnectService;
import dev.fitko.fitconnect.api.domain.model.attachment.Attachment;
import dev.fitko.fitconnect.api.domain.model.event.authtags.AuthenticationTags;
import dev.fitko.fitconnect.api.domain.model.event.problems.Problem;
import dev.fitko.fitconnect.api.domain.model.metadata.Metadata;
import dev.fitko.fitconnect.api.domain.model.reply.AcceptReply;
import dev.fitko.fitconnect.api.domain.model.reply.Reply;
import dev.fitko.fitconnect.api.domain.subscriber.ReceivedData;

import java.net.URI;
import java.util.Arrays;
import java.util.List;
import java.util.UUID;

public class ReceivedReply {

    private final transient FitConnectService fitConnectService;
    private final Reply reply;
    private final ReceivedData receivedData;
    private final Metadata metadata;
    private final List<Attachment> attachments;
    private final AuthenticationTags authenticationTags;


    public ReceivedReply(final FitConnectService fitConnectService, final Reply reply, final ReceivedData receivedData, final Metadata metadata, final List<Attachment> attachments, final AuthenticationTags authenticationTags) {
        this.fitConnectService = fitConnectService;
        this.reply = reply;
        this.receivedData = receivedData;
        this.metadata = metadata;
        this.attachments = attachments;
        this.authenticationTags = authenticationTags;
    }

    public void acceptReply(final Problem... problems) {
        fitConnectService.acceptReply(reply.getReplyId(), new AcceptReply(Arrays.asList(problems), authenticationTags));
    }

    public void rejectReply(final List<Problem> problems) {
        fitConnectService.rejectReply(reply.getReplyId(), problems);
    }

    /**
     * Access decrypted reply data.
     *
     * @return data as string
     */
    public String getDataAsString() {
        return receivedData.getData();
    }

    /**
     * Gets the reply data mime-type.
     *
     * @return mimetype string
     */
    public String getDataMimeType() {
        return receivedData.getMimeType().value();
    }

    /**
     * Gets the reply data schema reference.
     *
     * @return data schema as URI
     */
    public URI getDataSchemaUri() {
        return metadata.getContentStructure().getData().getSubmissionSchema().getSchemaUri();
    }

    /**
     * Access list of decrypted attachments.
     *
     * @return list of {@link Attachment}
     */
    public List<Attachment> getAttachments() {
        return attachments;
    }

    /**
     * Gets the reply {@link Metadata}.
     *
     * @return metadata
     */
    public Metadata getMetadata() {
        return metadata;
    }

    /**
     * Gets the reply caseId.
     *
     * @return caseId as UUID
     */
    public UUID getCaseId() {
        return reply.getCaseId();
    }

    /**
     * Gets the reply id.
     *
     * @return replyId as UUID
     */
    public UUID getReplyId() {
        return reply.getReplyId();
    }
}
