package dev.fitko.fitconnect.api.domain.sender.steps.encrypted;

import dev.fitko.fitconnect.api.domain.model.callback.Callback;
import dev.fitko.fitconnect.api.domain.sender.SendableEncryptedSubmission;

import java.net.URI;
import java.util.Map;
import java.util.UUID;

public interface EncryptedOptionalPropertiesStep {

    /**
     * Add a list of encrypted attachments to the submission to send.
     *
     * @param attachments map of attachment-uuid on encrypted attachment content
     *
     * @return next step to set more additional properties
     */
    EncryptedOptionalPropertiesStep addEncryptedAttachments(Map<UUID, String> attachments);

    /**
     * Add an encrypted attachment to the submission to send.
     *
     * @param id UUID of the attachment to add
     * @param content the encrypted attachment content
     * @return next step to set more additional properties
     */
    EncryptedOptionalPropertiesStep addEncryptedAttachment(UUID id, String content);

    /**
     * Set {@link Callback} to be informed about submission status updates asynchronous without polling.
     * @see <a href="https://docs.fitko.de/fit-connect/docs/details/callbacks#callback-url">FIT-Connect documentation on callbacks</a>
     *
     * @param callbackUri URI that should be called to notify the sender
     * @param callbackSecret Secret to verify the callback
     * @return next step to set more additional properties
     */
    EncryptedOptionalPropertiesStep setCallback(URI callbackUri, String callbackSecret);

    /**
     * Add the new submission to an existing case.
     *
     * @param caseID unique identifier of the submissions case
     * @return next step to set more additional properties
     */
    EncryptedOptionalPropertiesStep setCase(UUID caseID);

    /**
     * Constructs a new sendable encrypted submission from the data that is set.
     * @return SendableEncryptedSubmission
     */
    SendableEncryptedSubmission build();
}
